//===============================================================================
// Goal Book.
// Copyright  2009 Mark Brownsword. 
//===============================================================================

#region Using Statements
using System;
using System.ComponentModel;
using System.Windows.Controls;
using GoalBook.Infrastructure.Events;
using GoalBook.Infrastructure.Interfaces;
#endregion

namespace GoalBook.Goals.Views
{
    /// <summary>
    /// Interaction logic for GoalsDialogView.xaml
    /// </summary>    
    public partial class GoalsDialogView : UserControl, IErrorInfoDialogView
    {
        #region Declarations
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor.
        /// </summary>
        public GoalsDialogView()
        {            
            InitializeComponent();            
        }
        #endregion

        #region Events

        #region LevelChanged
        private static LevelChangedEventHandler _levelChangedEventHandler;        
        /// <summary>
        /// Event indicating that the Level property has changed.
        /// </summary>
        public event LevelChangedEventHandler LevelChanged
        {
            add { _levelChangedEventHandler = (LevelChangedEventHandler)Delegate.Combine(_levelChangedEventHandler, value); }
            remove { _levelChangedEventHandler = (LevelChangedEventHandler)Delegate.Remove(_levelChangedEventHandler, value); }
        }
        /// <summary>
        /// Raise the LevelChanged event.
        /// </summary>
        /// <param name="args">Event args.</param>
        private void OnLevelChanged(LevelChangedEventArgs args)
        {
            if (_levelChangedEventHandler != null)
            {
                _levelChangedEventHandler(args);                
            }
        }
        #endregion

        #region ErrorInfoChanged
        private static ErrorInfoChangedEventHandler _errorInfoChangedEventHandler;        
        /// <summary>
        /// Event indicating that the ErrorInfo property has changed.
        /// </summary>
        public event ErrorInfoChangedEventHandler ErrorInfoChanged
        {
            add { _errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Combine(_errorInfoChangedEventHandler, value); }
            remove { _errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Remove(_errorInfoChangedEventHandler, value); }
        }
        private void OnErrorInfoChanged(ErrorInfoChangedEventArgs args)
        {
            if (_errorInfoChangedEventHandler != null)
            {
                _errorInfoChangedEventHandler(args);
            }
        }
        #endregion

        #endregion

        #region EventHandlers
        /// <summary>
        /// Handle UserControl_Loaded event.
        /// </summary>        
        private void UserControl_Loaded(object sender, System.Windows.RoutedEventArgs e)
        {
            this.Focus();
            this.textBoxTitle.Focus();
        }
        /// <summary>
        /// Handle comboBoxLevel_SelectionChanged event.
        /// </summary>        
        private void ComboBoxLevel_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int result;
            if (comboBoxLevel.SelectedValue != null && int.TryParse(comboBoxLevel.SelectedValue.ToString(), out result))
            {
                OnLevelChanged(new LevelChangedEventArgs(result));
            }
        }
        #endregion        

        #region Public Members
        /// <summary>
        /// Raise ErrorInfo Changed event. 
        /// </summary>        
        public void RaiseErrorInfoChanged(IDataErrorInfo info)
        {
            OnErrorInfoChanged(new ErrorInfoChangedEventArgs(info));
        }        
        #endregion
    }
}
