﻿//===============================================================================
// Goal Book.
// Copyright © 2009 Mark Brownsword. 
//===============================================================================

#region Using Statements
using System;
using Csla;
using Csla.Validation;
#endregion

namespace GoalBook.Infrastructure
{
    /// <summary>
    /// Proxy type.
    /// </summary>
    [Serializable]
    public class KeyValueItem : BusinessBase<KeyValueItem>
    {
        #region Constants and Enums
        #endregion

        #region Inner Classes and Structures
        #endregion

        #region Delegates and Events
        #endregion

        #region Instance and Shared Fields
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor.
        /// </summary>
        public KeyValueItem()
        {
            MarkAsChild();

            ValidationRules.CheckRules();
        }
        /// <summary>
        /// Constructor.
        /// </summary>        
        public KeyValueItem(string key, string value, int order)
            : this()
        {
            this.Key = key;
            this.Value = value;
            this.Order = order;
                        
            base.MarkOld();
        }
        #endregion

        #region Properties
        public static PropertyInfo<string> KeyProperty = RegisterProperty(typeof(KeyValueItem), new PropertyInfo<string>("Key"));
        public static PropertyInfo<string> ValueProperty = RegisterProperty(typeof(KeyValueItem), new PropertyInfo<string>("Value"));
        public static PropertyInfo<int> OrderProperty = RegisterProperty(typeof(KeyValueItem), new PropertyInfo<int>("Order"));
       
        /// <summary>
        /// Reference to Key.
        /// </summary>
        public string Key
        {
            get
            {
                CanReadProperty(KeyProperty.Name, true);
                return GetProperty(KeyProperty);
            }
            set
            {
                CanWriteProperty(KeyProperty.Name, true);
                if (!GetProperty(KeyProperty).Equals(value))
                {
                    SetProperty(KeyProperty, value);
                    PropertyHasChanged(KeyProperty.Name);
                }
            }
        }
        /// <summary>
        /// Reference to Value.
        /// </summary>
        public string Value
        {
            get
            {
                CanReadProperty(ValueProperty.Name, true);
                return GetProperty(ValueProperty);
            }
            set
            {
                CanWriteProperty(ValueProperty.Name, true);
                if (!GetProperty(ValueProperty).Equals(value))
                {
                    SetProperty(ValueProperty, value);
                    PropertyHasChanged(ValueProperty.Name);
                }
            }
        }

        /// <summary>
        /// Reference to Order field.
        /// </summary>
        public int Order
        {
            get
            {
                CanReadProperty(OrderProperty.Name, true);
                return GetProperty(OrderProperty);
            }
            set
            {
                CanWriteProperty(OrderProperty.Name, true);
                if (!GetProperty(OrderProperty).Equals(value))
                {
                    SetProperty(OrderProperty, value);
                    PropertyHasChanged(OrderProperty.Name);
                }
            }
        }
        #endregion

        #region Private and Protected Methods               
        #endregion

        #region Public and internal Methods                
        #endregion

        #region Event Handlers        
        #endregion

        #region Base Class Overrides        
        /// <summary>
        /// Override for GetIdValue.
        /// </summary>        
        protected override object GetIdValue()
        {
            return GetProperty(KeyProperty);
        }
        #endregion        
    }
}
