﻿// <copyright file="ModuleSettingsInfo.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Infrastructure
{
    #region Using Statements
    using System.Xml;
    using System.Xml.Schema;
    using System.Xml.Serialization;
    #endregion

    /// <summary>
    /// Module Settings Info. Save module settings to Application Settings.
    /// </summary>
    public class ModuleSettingsInfo : IXmlSerializable
    {
        #region Constants and Enums
        /// <summary>
        /// Declaration for StartupLoadedField.
        /// </summary>
        private const string StartupLoadedField = "StartupLoaded";
        #endregion

        #region Instance and Shared Fields
        #endregion

        #region Constructors
        #endregion

        #region Delegates and Events
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets a value indicating whether module should StartupLoaded.
        /// </summary>
        public bool StartupLoaded { get; set; }
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// GetSchema. Implementation for IXmlSerializable.
        /// </summary>
        /// <returns>XmlSchema instance</returns>
        public XmlSchema GetSchema()
        {
            return null;
        }

        /// <summary>
        /// ReadXml. Implementation for IXmlSerializable.
        /// </summary>
        /// <param name="reader">XmlReader parameter</param>
        public virtual void ReadXml(XmlReader reader)
        {
            bool startupLoaded = false;
            if (bool.TryParse(reader.GetAttribute(StartupLoadedField), out startupLoaded))
            {
                this.StartupLoaded = startupLoaded;
            }
        }

        /// <summary>
        /// WriteXml. Implementation for IXmlSerializable.
        /// </summary>
        /// <param name="writer">XmlWriter parameter</param>
        public virtual void WriteXml(XmlWriter writer)
        {
            writer.WriteAttributeString(StartupLoadedField, this.StartupLoaded.ToString());            
        }
        #endregion

        #region Base Class Overrides
        #endregion

        #region Private and Protected Methods
        #endregion

        #region Event Handlers
        #endregion
    }
}
