﻿//===============================================================================
// Goal Book.
// Copyright © 2009 Mark Brownsword. 
//===============================================================================

#region Using Statements
using System;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
#endregion

namespace GoalBook.Infrastructure
{
    /// <summary>
    /// SyncTokenInfo type.
    /// </summary>
    [Serializable]
    public class SyncTokenInfo : IXmlSerializable
    {
        #region Declarations
        private const string TOKEN = "Token";
        private const string TOKEN_EXPIRES = "TokenExpires";
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor.
        /// </summary>        
        public SyncTokenInfo()
        {

        }
        /// <summary>
        /// Constructor.
        /// </summary>        
        public SyncTokenInfo(string token, DateTime tokenExpires)            
        {
            Token = token;
            TokenExpires = tokenExpires;
        }
        #endregion

        #region Properties
        /// <summary>
        /// Reference to Token.
        /// </summary>
        public string Token { get; set; }

        /// <summary>
        /// Reference to TokenExpires.
        /// </summary>
        public DateTime TokenExpires { get; set; }
        #endregion

        #region Public Members
        /// <summary>
        /// Clear all data.
        /// </summary>
        public void Clear()
        {
            this.Token = null;
            this.TokenExpires = DateTime.MinValue;            
        }
        #endregion

        #region IXmlSerializable Members
        public XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            Token = reader.GetAttribute(TOKEN);
            DateTime tokenExpires = DateTime.MinValue;
            if (DateTime.TryParse(reader.GetAttribute(TOKEN_EXPIRES), out tokenExpires))
            {
                TokenExpires = tokenExpires;
            }
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteAttributeString(TOKEN, Token);
            writer.WriteAttributeString(TOKEN_EXPIRES, TokenExpires.ToString());            
        }
        #endregion
    }
}
