﻿// <copyright file="ModuleCommands.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Notes
{
    #region Using Statements    
    using System.Windows;
    using GoalBook.Infrastructure;
    using Microsoft.Practices.Composite.Presentation.Commands;
    #endregion
    
    /// <summary>
    /// Commands for Notes Module.
    /// </summary>
    public class ModuleCommands : DependencyObject
    {
        #region Constants and Enums
        #endregion

        #region Instance and Shared Fields
        /// <summary>
        /// Declaration for CutCommandProperty.
        /// </summary>
        public static readonly DependencyProperty CutCommandProperty = DependencyProperty.Register(
            "CutCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for CopyCommandProperty.
        /// </summary>
        public static readonly DependencyProperty CopyCommandProperty = DependencyProperty.Register(
            "CopyCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for PasteCommandProperty.
        /// </summary>
        public static readonly DependencyProperty PasteCommandProperty = DependencyProperty.Register(
            "PasteCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for UndoCommandProperty.
        /// </summary>
        public static readonly DependencyProperty UndoCommandProperty = DependencyProperty.Register(
            "UndoCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for RedoCommandProperty.
        /// </summary>
        public static readonly DependencyProperty RedoCommandProperty = DependencyProperty.Register(
            "RedoCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for BoldCommandProperty.
        /// </summary>
        public static readonly DependencyProperty BoldCommandProperty = DependencyProperty.Register(
            "BoldCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for ItalicCommandProperty.
        /// </summary>
        public static readonly DependencyProperty ItalicCommandProperty = DependencyProperty.Register(
            "ItalicCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for LinkCommandProperty.
        /// </summary>
        public static readonly DependencyProperty LinkCommandProperty = DependencyProperty.Register(
            "LinkCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for OrderedCommandProperty.
        /// </summary>
        public static readonly DependencyProperty OrderedListCommandProperty = DependencyProperty.Register(
            "OrderedListCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for UnorderedCommandProperty.
        /// </summary>
        public static readonly DependencyProperty UnorderedListCommandProperty = DependencyProperty.Register(
            "UnorderedListCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for IndentCommandProperty.
        /// </summary>
        public static readonly DependencyProperty IndentCommandProperty = DependencyProperty.Register(
            "IndentCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));

        /// <summary>
        /// Declaration for OutdentCommandProperty.
        /// </summary>
        public static readonly DependencyProperty OutdentCommandProperty = DependencyProperty.Register(
            "OutdentCommand", typeof(DelegateCommand<CommandInfo>), typeof(ModuleCommands));
        #endregion

        #region Constructors
        #endregion

        #region Delegates and Events
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets CutCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> CutCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(CutCommandProperty); }
            set { SetValue(CutCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets CopyCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> CopyCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(CopyCommandProperty); }
            set { SetValue(CopyCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets PasteCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> PasteCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(PasteCommandProperty); }
            set { SetValue(PasteCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets UndoCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> UndoCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(UndoCommandProperty); }
            set { SetValue(UndoCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets RedoCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> RedoCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(RedoCommandProperty); }
            set { SetValue(RedoCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets BoldCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> BoldCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(BoldCommandProperty); }
            set { SetValue(BoldCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets ItalicCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> ItalicCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(ItalicCommandProperty); }
            set { SetValue(ItalicCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets LinkCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> LinkCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(LinkCommandProperty); }
            set { SetValue(LinkCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets OrderedListCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> OrderedListCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(OrderedListCommandProperty); }
            set { SetValue(OrderedListCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets UnorderedListCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> UnorderedListCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(UnorderedListCommandProperty); }
            set { SetValue(UnorderedListCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets IndentCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> IndentCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(IndentCommandProperty); }
            set { SetValue(IndentCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets OutdentCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> OutdentCommand
        {
            get { return (DelegateCommand<CommandInfo>)GetValue(OutdentCommandProperty); }
            set { SetValue(OutdentCommandProperty, value); }
        }
        #endregion

        #region Public and internal Methods
        #endregion

        #region Base Class Overrides
        #endregion

        #region Private and Protected Methods
        #endregion

        #region Event Handlers
        #endregion
    }    
}
