﻿// <copyright file="ModuleCommandsInfo.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Notes
{
    #region Using Statements    
    using System;
    using System.Windows.Input;
    using GoalBook.Infrastructure;
    using GoalBook.Notes.Properties;
    #endregion

    /// <summary>
    /// ModuleCommandsInfo class.
    /// </summary>
    public static class ModuleCommandsInfo
    {
        #region Constants and Enums
        #endregion

        #region Instance and Shared Fields        
        /// <summary>
        /// Declaration for CutCommandInfo.
        /// </summary>
        private static CommandInfo cutCommandInfo = new CommandInfo(
            Resources.ToolBarCutText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/cut.png", UriKind.Relative),
            new KeyGesture(Key.X, ModifierKeys.Control, Resources.ToolBarCutInputGestureText));
        
        /// <summary>
        /// Declaration for CopyCommandInfo.
        /// </summary>
        private static CommandInfo copyCommandInfo = new CommandInfo(
            Resources.ToolBarCopyText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/page_copy.png", UriKind.Relative),
            new KeyGesture(Key.C, ModifierKeys.Control, Resources.ToolBarCopyInputGestureText));
        
        /// <summary>
        /// Declaration for PasteCommandInfo.
        /// </summary>
        private static CommandInfo pasteCommandInfo = new CommandInfo(
            Resources.ToolBarPasteText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/paste_plain.png", UriKind.Relative),
            new KeyGesture(Key.V, ModifierKeys.Control, Resources.ToolBarPasteInputGestureText));
        
        /// <summary>
        /// Declaration for UndoCommandInfo.
        /// </summary>
        private static CommandInfo undoCommandInfo = new CommandInfo(
            Resources.ToolBarUndoText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/arrow_undo.png", UriKind.Relative),
            new KeyGesture(Key.Z, ModifierKeys.Control, Resources.ToolBarUndoInputGestureText));
        
        /// <summary>
        /// Declaration for RedoCommandInfo.
        /// </summary>
        private static CommandInfo redoCommandInfo = new CommandInfo(
            Resources.ToolBarRedoText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/arrow_redo.png", UriKind.Relative),
            new KeyGesture(Key.Y, ModifierKeys.Control, Resources.ToolBarRedoInputGestureText));
        
        /// <summary>
        /// Declaration for BoldCommandInfo.
        /// </summary>
        private static CommandInfo boldCommandInfo = new CommandInfo(
            Resources.ToolBarBoldText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/text_bold.png", UriKind.Relative),
            new KeyGesture(Key.B, ModifierKeys.Control, Resources.ToolBarBoldInputGestureText));

        /// <summary>
        /// Declaration for ItalicCommandInfo.
        /// </summary>
        private static CommandInfo italicCommandInfo = new CommandInfo(
            Resources.ToolBarItalicText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/text_italic.png", UriKind.Relative),
            new KeyGesture(Key.I, ModifierKeys.Control, Resources.ToolBarItalicInputGestureText));
        
        /// <summary>
        /// Declaration for LinkCommandInfo.
        /// </summary>
        private static CommandInfo linkCommandInfo = new CommandInfo(
            Resources.ToolBarLinkText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/world_link.png", UriKind.Relative),
            new KeyGesture(Key.K, ModifierKeys.Control, Resources.ToolBarLinkInputGestureText));
        
        /// <summary>
        /// Declaration for OrderedListCommandInfo.
        /// </summary>
        private static CommandInfo orderedListCommandInfo = new CommandInfo(
            Resources.ToolBarOrderedListText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/text_list_numbers.png", UriKind.Relative),
            null);
        
        /// <summary>
        /// Declaration for UnorderedListCommandInfo.
        /// </summary>
        private static CommandInfo unorderedListCommandInfo = new CommandInfo(
            Resources.ToolBarUnorderedListText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/text_list_bullets.png", UriKind.Relative),
            null);
        
        /// <summary>
        /// Declaration for indentCommandInfo.
        /// </summary>
        private static CommandInfo indentCommandInfo = new CommandInfo(
            Resources.ToolBarIndentText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/text_indent.png", UriKind.Relative),
            null);
        
        /// <summary>
        /// Declaration for outdentCommandInfo.
        /// </summary>
        private static CommandInfo outdentCommandInfo = new CommandInfo(
            Resources.ToolBarOutdentText,
            0,
            0,
            new Uri(@"/GoalBook.Notes;component/Resources/text_indent_remove.png", UriKind.Relative),
            null);        
        #endregion

        #region Constructors
        #endregion

        #region Delegates and Events
        #endregion

        #region Properties
        /// <summary>
        /// Gets CutCommandInfo.
        /// </summary>
        public static CommandInfo CutCommandInfo
        {
            get { return cutCommandInfo; }
        }

        /// <summary>
        /// Gets CopyCommandInfo.
        /// </summary>
        public static CommandInfo CopyCommandInfo
        {
            get { return copyCommandInfo; }
        }

        /// <summary>
        /// Gets PasteCommandInfo.
        /// </summary>
        public static CommandInfo PasteCommandInfo
        {
            get { return pasteCommandInfo; }
        }

        /// <summary>
        /// Gets UndoCommandInfo.
        /// </summary>
        public static CommandInfo UndoCommandInfo
        {
            get { return undoCommandInfo; }
        }

        /// <summary>
        /// Gets RedoCommandInfo.
        /// </summary>
        public static CommandInfo RedoCommandInfo
        {
            get { return redoCommandInfo; }
        }

        /// <summary>
        /// Gets BoldCommandInfo.
        /// </summary>
        public static CommandInfo BoldCommandInfo
        {
            get { return boldCommandInfo; }
        }

        /// <summary>
        /// Gets ItalicCommandInfo.
        /// </summary>
        public static CommandInfo ItalicCommandInfo
        {
            get { return italicCommandInfo; }
        }

        /// <summary>
        /// Gets LinkCommandInfo.
        /// </summary>
        public static CommandInfo LinkCommandInfo
        {
            get { return linkCommandInfo; }
        }

        /// <summary>
        /// Gets OrderListCommandInfo.
        /// </summary>
        public static CommandInfo OrderedListCommandInfo
        {
            get { return orderedListCommandInfo; }
        }

        /// <summary>
        /// Gets UnorderedListCommandInfo.
        /// </summary>
        public static CommandInfo UnorderedListCommandInfo
        {
            get { return ModuleCommandsInfo.unorderedListCommandInfo; }
        }

        /// <summary>
        /// Gets IndentCommandInfo.
        /// </summary>
        public static CommandInfo IndentCommandInfo
        {
            get { return indentCommandInfo; }
        }

        /// <summary>
        /// Gets OutdentCommandInfo.
        /// </summary>
        public static CommandInfo OutdentCommandInfo
        {
            get { return outdentCommandInfo; }
        }
        #endregion

        #region Public and internal Methods
        #endregion

        #region Base Class Overrides
        #endregion

        #region Private and Protected Methods
        #endregion

        #region Event Handlers
        #endregion
    }
}
