﻿// <copyright file="NotesDialogViewPresentationModel.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Notes.Views
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Windows;
    using GoalBook.Infrastructure;
    using GoalBook.Infrastructure.ObjectModel;
    using Microsoft.Practices.Composite.Presentation.Commands;
    #endregion

    /// <summary>
    /// Notes Dialog View PresentationModel class.
    /// </summary>
    public class NotesDialogViewPresentationModel : DependencyObject
    {
        #region Constants and Enums
        #endregion

        #region Instance and Shared Fields
        /// <summary>
        /// Declaration for commands.
        /// </summary>
        private ModuleCommands commands;
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the NotesDialogViewPresentationModel class.
        /// </summary>
        /// <param name="commands">ModuleCommands parameter</param>
        public NotesDialogViewPresentationModel(ModuleCommands commands)
        {
            this.commands = commands;
        }
        #endregion

        #region Delegates and Events

        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets Edit Note.
        /// </summary>
        public Note EditNote { get; set; }

        /// <summary>
        /// Gets or sets FolderList. Folder Reference data.
        /// </summary>
        public Dictionary<Guid, string> FolderList { get; set; }

        /// <summary>
        /// Gets or sets CutCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> CutCommand
        {
            get { return this.commands.CutCommand; }
            set { this.commands.CutCommand = value; }
        }

        /// <summary>
        /// Gets CutCommandInfo for CutCommand.
        /// </summary>
        public CommandInfo CutCommandInfo
        {
            get { return ModuleCommandsInfo.CutCommandInfo; }
        }

        /// <summary>
        /// Gets or sets CopyCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> CopyCommand
        {
            get { return this.commands.CopyCommand; }
            set { this.commands.CopyCommand = value; }
        }

        /// <summary>
        /// Gets CopyCommandInfo for CopyCommand.
        /// </summary>
        public CommandInfo CopyCommandInfo
        {
            get { return ModuleCommandsInfo.CopyCommandInfo; }
        }

        /// <summary>
        /// Gets or sets PasteCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> PasteCommand
        {
            get { return this.commands.PasteCommand; }
            set { this.commands.PasteCommand = value; }
        }

        /// <summary>
        /// Gets PasteCommandInfo for PasteCommand.
        /// </summary>
        public CommandInfo PasteCommandInfo
        {
            get { return ModuleCommandsInfo.PasteCommandInfo; }
        }

        /// <summary>
        /// Gets or sets UndoCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> UndoCommand
        {
            get { return this.commands.UndoCommand; }
            set { this.commands.UndoCommand = value; }
        }

        /// <summary>
        /// Gets UndoCommandInfo for UndoCommand.
        /// </summary>
        public CommandInfo UndoCommandInfo
        {
            get { return ModuleCommandsInfo.UndoCommandInfo; }
        }

        /// <summary>
        /// Gets or sets RedoCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> RedoCommand
        {
            get { return this.commands.RedoCommand; }
            set { this.commands.RedoCommand = value; }
        }

        /// <summary>
        /// Gets RedoCommandInfo for RedoCommand.
        /// </summary>
        public CommandInfo RedoCommandInfo
        {
            get { return ModuleCommandsInfo.RedoCommandInfo; }
        }

        /// <summary>
        /// Gets or sets BoldCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> BoldCommand
        {
            get { return this.commands.BoldCommand; }
            set { this.commands.BoldCommand = value; }
        }

        /// <summary>
        /// Gets BoldCommandInfo for BoldCommand.
        /// </summary>
        public CommandInfo BoldCommandInfo
        {
            get { return ModuleCommandsInfo.BoldCommandInfo; }
        }

        /// <summary>
        /// Gets or sets ItalicCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> ItalicCommand
        {
            get { return this.commands.ItalicCommand; }
            set { this.commands.ItalicCommand = value; }
        }

        /// <summary>
        /// Gets ItalicCommandInfo for ItalicCommand.
        /// </summary>
        public CommandInfo ItalicCommandInfo
        {
            get { return ModuleCommandsInfo.ItalicCommandInfo; }
        }

        /// <summary>
        /// Gets or sets LinkCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> LinkCommand
        {
            get { return this.commands.LinkCommand; }
            set { this.commands.LinkCommand = value; }
        }

        /// <summary>
        /// Gets LinkCommandInfo for LinkCommand.
        /// </summary>
        public CommandInfo LinkCommandInfo
        {
            get { return ModuleCommandsInfo.LinkCommandInfo; }
        }

        /// <summary>
        /// Gets or sets OrderedListCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> OrderedListCommand
        {
            get { return this.commands.OrderedListCommand; }
            set { this.commands.OrderedListCommand = value; }
        }

        /// <summary>
        /// Gets OrderedListCommandInfo for OrderedListCommand.
        /// </summary>
        public CommandInfo OrderedListCommandInfo
        {
            get { return ModuleCommandsInfo.OrderedListCommandInfo; }
        }

        /// <summary>
        /// Gets or sets UnorderedListCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> UnorderedListCommand
        {
            get { return this.commands.UnorderedListCommand; }
            set { this.commands.UnorderedListCommand = value; }
        }

        /// <summary>
        /// Gets UnorderedListCommandInfo for UnorderedListCommand.
        /// </summary>
        public CommandInfo UnorderedListCommandInfo
        {
            get { return ModuleCommandsInfo.UnorderedListCommandInfo; }
        }

        /// <summary>
        /// Gets or sets IndentCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> IndentCommand
        {
            get { return this.commands.IndentCommand; }
            set { this.commands.IndentCommand = value; }
        }

        /// <summary>
        /// Gets IndentCommandInfo for IndentCommand.
        /// </summary>
        public CommandInfo IndentCommandInfo
        {
            get { return ModuleCommandsInfo.IndentCommandInfo; }
        }

        /// <summary>
        /// Gets or sets OutdentCommand.
        /// </summary>
        public DelegateCommand<CommandInfo> OutdentCommand
        {
            get { return this.commands.OutdentCommand; }
            set { this.commands.OutdentCommand = value; }
        }

        /// <summary>
        /// Gets OutdentCommandInfo for OutdentCommand.
        /// </summary>
        public CommandInfo OutdentCommandInfo
        {
            get { return ModuleCommandsInfo.OutdentCommandInfo; }
        }

        #endregion

        #region Public and internal Methods
        #endregion

        #region Base Class Overrides
        #endregion

        #region Private and Protected Methods
        #endregion

        #region Event Handlers
        #endregion
    }
}
