//===============================================================================
// Goal Book.
// Copyright  2009 Mark Brownsword. 
//===============================================================================

#region Using Statements
using System.Windows;
using GoalBook.Goals;
using GoalBook.Infrastructure.Interfaces;
using GoalBook.Notes;
using GoalBook.Shell.Commands;
using GoalBook.Shell.Misc;
using GoalBook.Shell.Modules;
using GoalBook.Shell.Services;
using GoalBook.Shell.Windows;
using GoalBook.Tasks;
using Microsoft.Practices.Composite.Logging;
using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Composite.UnityExtensions;
using Microsoft.Practices.Unity;
#endregion

namespace GoalBook.Shell
{
    internal class Bootstrapper : UnityBootstrapper
    {
        #region Constants and Enums
        #endregion

        #region Inner Classes and Structures
        #endregion

        #region Delegates and Events
        #endregion

        #region Instance and Shared Fields
        private ApplicationSettings _applicationSettings;
        private EventLogging _eventLogging;
        private MainPresenter _presenter;           
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor.
        /// </summary>
        public Bootstrapper(ApplicationSettings applicationSettings, EventLogging eventLogging)
        {
            _applicationSettings = applicationSettings;
            _eventLogging = eventLogging;
        }
        #endregion

        #region Properties
        #endregion

        #region Private and Protected Methods
        /// <summary>
        /// Get Module Catalog. Statically loaded modules.
        /// </summary>        
        protected override IModuleCatalog GetModuleCatalog()
        {
            return new ModuleCatalog()
                .AddModule(typeof(TasksModule))
                .AddModule(typeof(NotesModule))
                .AddModule(typeof(GoalsModule))
                .AddModule(typeof(SearchModule));                                 
        }
        /// <summary>
        /// Configure Container.
        /// </summary>
        protected override void ConfigureContainer()
        {     
            // Register services (using ContainerControlledLifetimeManager to create singleton instances).
            Container.RegisterType<IShellView, Main>(new ContainerControlledLifetimeManager());
            Container.RegisterType<INavigationService, NavigationService>(new ContainerControlledLifetimeManager());
            Container.RegisterType<IDialogService, DialogService>(new ContainerControlledLifetimeManager());
            Container.RegisterType<IGlobalCommandsProxy, GlobalCommandsProxy>(new ContainerControlledLifetimeManager());
            Container.RegisterType<IPersistenceService, IsolatedStorageService>(new ContainerControlledLifetimeManager());
            Container.RegisterType<IPrintService, PrintService>(new ContainerControlledLifetimeManager());
            
            // Register existing instances as services
            Container.RegisterInstance<ISettingsService>(_applicationSettings, new ContainerControlledLifetimeManager());
            Container.RegisterInstance<ILoggerService>(_eventLogging, new ContainerControlledLifetimeManager());

            base.ConfigureContainer();
        }
        /// <summary>
        /// Create Shell. Use the Unity Container to create an instance of Shell Presenter. The Unity
        /// Container inspects the ShellPresenter contructor and injects the required types. Also 
        /// Set presenter model class as DataContext for the View. This makes everything 
        /// in the model (properties, commands etc) class available for XAML binding.
        /// </summary>        
        protected override DependencyObject CreateShell()
        {            
            _presenter = Container.Resolve<MainPresenter>();                        
            _presenter.View.DataContext = _presenter.Model;
            _presenter.View.InitialiseSettings(_applicationSettings);
                        
            return _presenter.View as DependencyObject;               
        }        
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// ShowShell.
        /// </summary>
        public void ShowShell()
        {                        
            _presenter.View.ShowView();            
        }        
        #endregion

        #region Event Handlers
        #endregion

        #region Base Class Overrides
        /// <summary>
        /// Reference to LoggerService. Used by the Container to log information and exceptions.
        /// </summary>
        protected override ILoggerFacade LoggerFacade
        {
            get { return _eventLogging as ILoggerFacade; }
        }
        #endregion
    }
}
