﻿// <copyright file="GlobalCommandsInfo.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Shell.Commands
{
    #region Using Statements
    using System;
    using System.Windows.Input;
    using GoalBook.Infrastructure;
    using GoalBook.Infrastructure.Enums;
    using GoalBook.Shell.Properties;
    using InfrastructureConstants = GoalBook.Infrastructure.Constants.MenuConstants;
    #endregion
    
    /// <summary>
    /// Class for Global CommandsInfo.
    /// </summary>
    public static class GlobalCommandsInfo
    {
        /// <summary>
        /// Declaration for printCommandInfo.
        /// </summary>
        private static CommandInfo printCommandInfo = new CommandInfo(
            Resources.MenuPrintText,
            4,
            ParentMenuType.Module,
            new Uri(InfrastructureConstants.MENU_PRINT_IMAGE_URI),
            new KeyGesture(Key.P, ModifierKeys.Control, Resources.MenuPrintInputGestureText));
        
        /// <summary>
        /// Declaration for undoCommandInfo.
        /// </summary>
        private static CommandInfo undoCommandInfo = new CommandInfo(
            Resources.MenuUndoText,
            0,
            ParentMenuType.Edit,
            new Uri(InfrastructureConstants.MENU_UNDO_IMAGE_URI),
            new KeyGesture(Key.Z, ModifierKeys.Control, Resources.MenuUndoInputGestureText));

        /// <summary>
        /// Declaration for deleteCommandInfo.
        /// </summary>
        private static CommandInfo deleteCommandInfo = new CommandInfo(
            Resources.MenuDeleteText,
            2,
            ParentMenuType.Edit,
            new Uri(InfrastructureConstants.MENU_DELETE_IMAGE_URI),
            new KeyGesture(Key.Delete, ModifierKeys.None, Resources.MenuDeleteInputGestureText));
        
        /// <summary>
        /// Gets reference to PrintCommandInfo.
        /// </summary>
        public static CommandInfo PrintCommandInfo
        {
            get { return GlobalCommandsInfo.printCommandInfo; }
        }

        /// <summary>
        /// Gets reference to UndoCommandInfo.
        /// </summary>
        public static CommandInfo UndoCommandInfo
        {
            get { return GlobalCommandsInfo.undoCommandInfo; }
        }

        /// <summary>
        /// Gets reference to DeleteCommandInfo.
        /// </summary>
        public static CommandInfo DeleteCommandInfo
        {
            get { return GlobalCommandsInfo.deleteCommandInfo; }
        }        
    }    
}
